@@ -41,7 +41,7 @@
 
 static char * infilename;
 static int begun = 0;
-static FishSoundInfo fsinfo;
+static long actual_frames_decoded = 0;
 
 static void
 read_comments (FishSound * fsound)
@@ -70,6 +70,8 @@ decoded (FishSound * fsound, float ** pcm, long frames, void * user_data)
     begun = 1;
   }
 
+  actual_frames_decoded += frames;
+
   return 0;
 }
 
@@ -78,9 +80,13 @@ read_packet (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
 {
   FishSound * fsound = (FishSound *)user_data;
 
+  if (op->e_o_s) {
+    fish_sound_prepare_truncation (fsound, op->e_o_s, op->granulepos);
+  }
+
   fish_sound_decode (fsound, op->packet, op->bytes);
 
-  return 0;
+  return OGGZ_CONTINUE;
 }
 
 int
@@ -93,13 +99,13 @@ main (int argc, char ** argv)
   if (argc < 2) {
     printf ("usage: %s infilename\n", argv[0]);
     printf ("*** FishSound example program. ***\n");
-    printf ("Read comments from an Ogg Speex or Ogg Vorbis file.\n");
+    printf ("Display information about an Ogg FLAC, Speex or Ogg Vorbis file.\n");
     exit (1);
   }
 
   infilename = argv[1];
 
-  fsound = fish_sound_new (FISH_SOUND_DECODE, &fsinfo);
+  fsound = fish_sound_new (FISH_SOUND_DECODE, NULL);
   fish_sound_set_interleave (fsound, 1);
   fish_sound_set_decoded_float_ilv (fsound, decoded, NULL);
 
@@ -114,6 +120,9 @@ main (int argc, char ** argv)
 
   oggz_close (oggz);
 
+  printf ("%ld frames decoded\n", actual_frames_decoded);
+  printf ("%ld frames reported\n", fish_sound_get_frameno (fsound));
+
   fish_sound_delete (fsound);
   
   exit (0);
