@@ -45,8 +45,10 @@
 #include "private.h"
 #include "convert.h"
 
-/*#define DEBUG*/
-/*#define DEBUG_VERBOSE*/
+#define DEBUG_VERBOSE 2
+/* #define DEBUG */
+/* #define DEBUG_LEVEL DEBUG_VERBOSE */
+#include "debug.h"
 
 #if HAVE_FLAC
 
@@ -59,7 +61,7 @@ typedef struct _FishSoundFlacInfo {
   FLAC__StreamEncoder *fse;
   unsigned char * buffer;
   char header;
-  long bufferlength;
+  unsigned long bufferlength;
   unsigned long packetno;
   struct {
     unsigned char major, minor;
@@ -82,9 +84,7 @@ fish_sound_flac_identify (unsigned char * buf, long bytes)
   if (bytes < 8) return FISH_SOUND_UNKNOWN;
   if (buf[0] != 0x7f) return FISH_SOUND_UNKNOWN;
   if (!strncmp ((char *)buf+1, "FLAC", 4)) {
-#ifdef DEBUG
-    printf("fish_sound_flac_identify: flac found\n");
-#endif
+    debug_printf(1, "flac found");
     /* if only a short buffer was passed, do a weak identify */
     if (bytes == 8) return FISH_SOUND_FLAC;
 
@@ -106,28 +106,22 @@ fs_flac_command (FishSound * fsound, int command, void * data, int datasize)
 #if FS_DECODE
 static FLAC__StreamDecoderReadStatus
 fs_flac_read_callback(const FLAC__StreamDecoder *decoder,
-                      FLAC__byte buffer[], unsigned *bytes,
+                      FLAC__byte buffer[], unsigned int *bytes,
                       void *client_data)
 {
   FishSound* fsound = (FishSound*)client_data;
   FishSoundFlacInfo* fi = (FishSoundFlacInfo *)fsound->codec_data;
-#ifdef DEBUG_VERBOSE
-  printf("fs_flac_read_callback: IN\n");
-#endif
+  debug_printf(DEBUG_VERBOSE, "fs_flac_read_callback: IN");
   if (fi->bufferlength > *bytes) {
-#ifdef DEBUG
-    printf("fs_flac_read_callback: too much data\n");
-#endif
+    debug_printf(1, "too much data");
     return FLAC__STREAM_DECODER_READ_STATUS_ABORT;
   } else if (fi->bufferlength < 1) {
-#ifdef DEBUG
-    printf("fs_flac_read_callback: no data, %ld\n",fi->bufferlength);
-#endif
+    debug_printf(1, "no data, %ld",fi->bufferlength);
     return FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM;
   }
 
   memcpy(buffer, fi->buffer, fi->bufferlength);
-  *bytes = fi->bufferlength;
+  *bytes = (unsigned int)fi->bufferlength;
   fi->bufferlength = 0;
   return FLAC__STREAM_DECODER_READ_STATUS_CONTINUE;
 }
@@ -141,13 +135,12 @@ fs_flac_write_callback(const FLAC__StreamDecoder *decoder,
   FishSound* fsound = (FishSound*)client_data;
   FishSoundFlacInfo* fi = (FishSoundFlacInfo *)fsound->codec_data;
   int i, j, channels, blocksize, offset;
+  float * ipcm;
 
   channels = frame->header.channels;
   blocksize = frame->header.blocksize;
 
-#ifdef DEBUG_VERBOSE
-  printf("fs_flac_write_callback: IN, blocksize %d\n", blocksize);
-#endif
+  debug_printf(DEBUG_VERBOSE, "IN, blocksize %d", blocksize);
 
   fsound->frameno += blocksize;
 
@@ -158,7 +151,10 @@ fs_flac_write_callback(const FLAC__StreamDecoder *decoder,
 	FishSoundDecoded_FloatIlv dfi;
 	float* retpcm;
 
-	fi->ipcm = realloc(fi->ipcm, sizeof(float) * channels * blocksize);
+        if ((ipcm = realloc(fi->ipcm, sizeof(float) * channels * blocksize)) == NULL)
+          return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
+
+	fi->ipcm = ipcm;
 	retpcm = (float*) fi->ipcm;
 	for (i = 0; i < blocksize; i++) {
 	  offset = i * channels;
@@ -173,7 +169,9 @@ fs_flac_write_callback(const FLAC__StreamDecoder *decoder,
 	float *d; /* de-interleave dest */
 
         for (j = 0; j < channels; j++) {
-	  fi->pcm_out[j] = realloc(fi->pcm_out[j], sizeof(float) * blocksize);
+          if ((ipcm = realloc(fi->pcm_out[j], sizeof(float) * blocksize)) == NULL)
+            return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
+	  fi->pcm_out[j] = ipcm;
         }
 	for (i = 0; i < blocksize; i++)
 	  for (j = 0; j < channels; j++) {
@@ -194,23 +192,18 @@ fs_flac_meta_callback(const FLAC__StreamDecoder *decoder,
 {
   FishSound* fsound = (FishSound*)client_data;
   /*  FishSoundFlacInfo* fi = (FishSoundFlacInfo *)fsound->codec_data; */
-#ifdef DEBUG
-  printf("fs_flac_meta_callback: IN\n");
-#endif
+  debug_printf(1, "IN");
+
   switch (metadata->type) {
   case FLAC__METADATA_TYPE_STREAMINFO:
-#ifdef DEBUG
-    printf("fs_flac_meta_callback: channels %d, samplerate %d\n",
+    debug_printf(1, "channels %d, samplerate %d",
            metadata->data.stream_info.channels,
            metadata->data.stream_info.sample_rate);
-#endif
     fsound->info.channels = metadata->data.stream_info.channels;
     fsound->info.samplerate = metadata->data.stream_info.sample_rate;
     break;
   default:
-#ifdef DEBUG
-    printf("fs_flac_meta_callback: not yet implemented type\n");
-#endif
+    debug_printf(1, "not yet implemented type");
     break;
   }
 }
@@ -220,9 +213,7 @@ fs_flac_error_callback(const FLAC__StreamDecoder *decoder,
                        FLAC__StreamDecoderErrorStatus status,
                        void *client_data)
 {
-#ifdef DEBUG
-  printf("fs_flac_error_callback: IN\n");
-#endif
+  debug_printf(1, "IN");
   fprintf(stderr, "FLAC ERROR: %s\n", FLAC__StreamDecoderErrorStatusString[status]);
 }
 #endif
@@ -231,27 +222,23 @@ static void*
 fs_flac_decode_header (FishSound * fsound, unsigned char *buf, long bytes)
 {
   FishSoundFlacInfo *fi = fsound->codec_data;
+
+  if (bytes < 9) return NULL;
   if (buf[0] != 0x7f) return NULL;
   if (strncmp((char *)buf+1, "FLAC", 4) != 0) return NULL;
   fi->version.major = buf[5];
   fi->version.minor = buf[6];
-#ifdef DEBUG
-  printf("fs_flac_decode_header : Flac Ogg Mapping Version: %d.%d\n",
+  debug_printf(1, "Flac Ogg Mapping Version: %d.%d",
          fi->version.major, fi->version.minor);
-#endif
   fi->header_packets = buf[7] << 8 | buf[8];
-#ifdef DEBUG
-  printf("fs_flac_decode_header: Number of Header packets: %d\n",
-         fi->header_packets);
-#endif
+  debug_printf(1, "Number of Header packets: %d", fi->header_packets);
 
   if ((fi->fsd = FLAC__stream_decoder_new()) == NULL) {
-#ifdef DEBUG
-    printf ("fs_flac_decode_header: unable to create new stream_decoder\n");
-#endif
+    debug_printf (1, "unable to create new stream_decoder");
     return NULL;
   }
 
+#if defined (HAVE_FLAC_1_1_2)
   FLAC__stream_decoder_set_read_callback(fi->fsd, fs_flac_read_callback);
   FLAC__stream_decoder_set_write_callback(fi->fsd, fs_flac_write_callback);
   FLAC__stream_decoder_set_metadata_callback(fi->fsd, fs_flac_meta_callback);
@@ -260,6 +247,21 @@ fs_flac_decode_header (FishSound * fsound, unsigned char *buf, long bytes)
 
   if (FLAC__stream_decoder_init(fi->fsd) != FLAC__STREAM_DECODER_SEARCH_FOR_METADATA)
     return NULL;
+#elif defined (HAVE_FLAC_1_1_3)
+  if (FLAC__stream_decoder_init_stream
+      (fi->fsd,
+       fs_flac_read_callback,
+       NULL, /* seek callback */
+       NULL, /* tell callback */
+       NULL, /* length callback */
+       NULL, /* EOF callback */
+       fs_flac_write_callback,
+       fs_flac_meta_callback,
+       fs_flac_error_callback,
+       fsound
+       ) != FLAC__STREAM_DECODER_SEARCH_FOR_METADATA)
+     return NULL;
+#endif
 
   return fi->fsd;
 }
@@ -269,27 +271,23 @@ fs_flac_decode (FishSound * fsound, unsigned char * buf, long bytes)
 {
   FishSoundFlacInfo *fi = fsound->codec_data;
 
-#ifdef DEBUG_VERBOSE
-  printf("fs_flac_decode: IN, fi->packetno = %ld\n", fi->packetno);
-#endif
+  debug_printf(DEBUG_VERBOSE, "IN, fi->packetno = %ld", fi->packetno);
 
   if (fi->packetno == 0) {
     if (fs_flac_decode_header (fsound, buf, bytes) == NULL) {
-#ifdef DEBUG
-      printf("fs_flac_decode: Error reading header\n");
-#endif
+      debug_printf(1, "Error reading header");
       return -1;
     }
-    fi->buffer = fs_malloc(sizeof(unsigned char)*bytes);
+    if ((fi->buffer = fs_malloc(sizeof(unsigned char)*bytes)) == NULL)
+      return FISH_SOUND_ERR_OUT_OF_MEMORY;
+
     memcpy(fi->buffer, buf+9, bytes-9);
     fi->bufferlength = bytes-9;
   }
   else if (fi->packetno <= fi->header_packets){
-    unsigned char* tmp = fs_malloc(sizeof(unsigned char)*(fi->bufferlength+bytes));
-#ifdef DEBUG
-    printf("fs_flac_decode: handling header (fi->header_packets = %d)\n",
-           fi->header_packets);
-#endif
+    unsigned char* tmp;
+    debug_printf(1, "handling header (fi->header_packets = %d)",
+                 fi->header_packets);
 
 #if 0
     if (fi->packetno ==  1) fish_sound_comments_decode (fsound, buf, bytes);
@@ -297,29 +295,46 @@ fs_flac_decode (FishSound * fsound, unsigned char * buf, long bytes)
 
     if ((buf[0] & 0x7) == 4) {
       int len = (buf[1]<<16) + (buf[2]<<8) + buf[3];
-#ifdef DEBUG
-      printf ("fs_flac_decode: got vorbiscomments len %d\n", len);
-#endif
-      fish_sound_comments_decode (fsound, buf+4, len);
+      debug_printf (1, "got vorbiscomments len %d", len);
+
+      if (fish_sound_comments_decode (fsound, buf+4, len) == FISH_SOUND_ERR_OUT_OF_MEMORY) {
+        fi->packetno++;
+        return FISH_SOUND_ERR_OUT_OF_MEMORY;
+      }
     }
 
+    if ((tmp = fs_malloc(sizeof(unsigned char)*(fi->bufferlength+bytes))) == NULL)
+      return FISH_SOUND_ERR_OUT_OF_MEMORY;
+
     memcpy(tmp, fi->buffer, fi->bufferlength);
     memcpy(tmp+fi->bufferlength, buf, bytes);
     fi->bufferlength += bytes;
     fs_free(fi->buffer);
     fi->buffer = tmp;
     if (fi->packetno == fi->header_packets) {
-      FLAC__stream_decoder_process_until_end_of_metadata(fi->fsd);
+      if (FLAC__stream_decoder_process_until_end_of_metadata(fi->fsd) == false) {
+        goto dec_err;
+      }
       fs_free(fi->buffer);
     }
   } else {
     fi->buffer = buf;
     fi->bufferlength = bytes;
-    FLAC__stream_decoder_process_single(fi->fsd);
+    if (FLAC__stream_decoder_process_single(fi->fsd) == false) {
+      goto dec_err;
+    }
   }
   fi->packetno++;
 
   return 0;
+
+dec_err:
+    switch (FLAC__stream_decoder_get_state(fi->fsd)) {
+    case FLAC__STREAM_DECODER_MEMORY_ALLOCATION_ERROR:
+      return FISH_SOUND_ERR_OUT_OF_MEMORY;
+    default:
+      return FISH_SOUND_ERR_GENERIC;
+    }
 }
 #else /* !FS_DECODE */
 
@@ -337,10 +352,10 @@ fs_flac_enc_write_callback(const FLAC__StreamEncoder *encoder,
 {
   FishSound* fsound = (FishSound*)client_data;
   FishSoundFlacInfo *fi = fsound->codec_data;
-#ifdef DEBUG
-  printf("fs_flac_enc_write_callback: IN\n");
-  printf("fs_flac_enc_write_callback: bytes: %d, samples: %d\n", bytes, samples);
-#endif
+
+  debug_printf(1, "IN");
+  debug_printf(1, "bytes: %d, samples: %d", bytes, samples);
+
   if (fsound->callback.encoded) {
     FishSoundEncoded encoded = (FishSoundEncoded) fsound->callback.encoded;
     if (fi->packetno == 0 && fi->header <= 1) {
@@ -349,11 +364,12 @@ fs_flac_enc_write_callback(const FLAC__StreamEncoder *encoder,
          * and a STREAMINFO block. Prepend the FLAC Ogg mapping header,
          * as described in http://flac.sourceforge.net/ogg_mapping.html.
          */
-#ifdef DEBUG
-        printf("fs_flac_enc_write_callback: generating FLAC header packet: "
-               "%c%c%c%c\n", buffer[0], buffer[1], buffer[2], buffer[3]);
-#endif
-	fi->buffer = (unsigned char*)malloc(sizeof(unsigned char)*(bytes+9));
+        debug_printf(1, "generating FLAC header packet: %c%c%c%c",
+                     buffer[0], buffer[1], buffer[2], buffer[3]);
+
+	if ((fi->buffer = (unsigned char*)fs_malloc(sizeof(unsigned char)*(bytes+9))) == NULL)
+          return FLAC__STREAM_ENCODER_WRITE_STATUS_FATAL_ERROR;
+
 	fi->buffer[0] = 0x7f;
 	fi->buffer[1] = 0x46; /* 'F' */
 	fi->buffer[2] = 0x4c; /* 'L' */
@@ -371,7 +387,11 @@ fs_flac_enc_write_callback(const FLAC__StreamEncoder *encoder,
         /* Make a temporary copy of the metadata header to pass to the user
          * callback.
          */
-	unsigned char* tmp = (unsigned char*)malloc(sizeof(unsigned char)*(bytes+fi->bufferlength));
+	unsigned char* tmp;
+
+        if ((tmp = (unsigned char*)fs_malloc(sizeof(unsigned char)*(bytes+fi->bufferlength))) == NULL)
+          return FLAC__STREAM_ENCODER_WRITE_STATUS_FATAL_ERROR;
+
 	memcpy (tmp, fi->buffer, fi->bufferlength);
 	memcpy (tmp+fi->bufferlength, buffer, bytes);
 	fs_free(fi->buffer);
@@ -398,25 +418,20 @@ fs_flac_enc_meta_callback(const FLAC__StreamEncoder *encoder,
 {
   /* FishSound* fsound = (FishSound*)client_data; */
   /* FishSoundFlacInfo* fi = (FishSoundFlacInfo *)fsound->codec_data; */
-#ifdef DEBUG
-  printf("fs_flac_enc_meta_callback: IN\n");
-#endif
+  debug_printf(1, "IN");
+
   switch (metadata->type) {
   case FLAC__METADATA_TYPE_STREAMINFO:
-#ifdef DEBUG
-    printf("fs_flac_enc_meta_callback: channels %d, samplerate %d\n",
+    debug_printf(1, "channels %d, samplerate %d",
            metadata->data.stream_info.channels,
            metadata->data.stream_info.sample_rate);
-#endif
     /*
     fsound->info.channels = metadata->data.stream_info.channels;
     fsound->info.samplerate = metadata->data.stream_info.sample_rate;
     */
     break;
   default:
-#ifdef DEBUG
-    printf("fs_flac_enc_meta_callback: metadata type not yet implemented\n");
-#endif
+    debug_printf(1, "metadata type not yet implemented");
     break;
   }
 
@@ -462,7 +477,8 @@ fs_flac_encode_vcentry (const FishSoundComment * comment)
     length += value_len + 1;
   }
 
-  entry = fs_malloc (length);
+  if ((entry = fs_malloc (length)) == NULL)
+    return NULL;
 
   /* We assume that comment->name, value are NUL terminated, as they were
    * produced by our own comments.c */
@@ -482,9 +498,10 @@ static FLAC__StreamMetadata *
 fs_flac_encode_vorbiscomments (FishSound * fsound)
 {
   FishSoundFlacInfo * fi = fsound->codec_data;
-  FLAC__StreamMetadata * metadata;
+  FLAC__StreamMetadata * metadata = NULL;
   const FishSoundComment * comment;
-  unsigned int i=0, length=0, total_length;
+  unsigned int length=0, total_length;
+  int i=0;
   FLAC__VCEntry * comments;
 
   /* libFLAC seems to require us to know the total length of the generated
@@ -502,11 +519,13 @@ fs_flac_encode_vorbiscomments (FishSound * fsound)
 
   if (length == 0) return NULL;
 
-  comments = (FLAC__VCEntry *)fs_malloc (sizeof(FLAC__VCEntry) * length);
+  if ((comments = (FLAC__VCEntry *)fs_malloc (sizeof(FLAC__VCEntry) * length)) == NULL)
+    goto encode_vc_oom;
   
   for (comment = fish_sound_comment_first (fsound); comment;
        comment = fish_sound_comment_next (fsound, comment)) {
-    comments[i].entry = fs_flac_encode_vcentry (comment);
+    if ((comments[i].entry = fs_flac_encode_vcentry (comment)) == NULL) {
+    }
     comments[i].length = strlen((char *)comments[i].entry);
 
     /* In the generated vorbiscomment data, each entry is preceded by a
@@ -515,7 +534,9 @@ fs_flac_encode_vorbiscomments (FishSound * fsound)
     i++;
   }
 
-  metadata = (FLAC__StreamMetadata *) fs_malloc (sizeof (*metadata));
+  if ((metadata = (FLAC__StreamMetadata *) fs_malloc (sizeof (*metadata))) == NULL)
+    goto encode_vc_oom;
+
   metadata->type = FLAC__METADATA_TYPE_VORBIS_COMMENT;
   metadata->is_last = true;
   metadata->length = total_length;
@@ -527,6 +548,21 @@ fs_flac_encode_vorbiscomments (FishSound * fsound)
   fi->enc_vc_metadata = metadata;
 
   return metadata;
+
+encode_vc_oom:
+  if (metadata != NULL)
+    fs_free (metadata);
+
+  /* Unwind allocated comment entries */
+  for (i--; i >= 0; i--) {
+    if (comments[i].entry != NULL)
+      fs_free (comments[i].entry);
+  }
+
+  if (comments != NULL)
+    fs_free (comments);
+
+  return NULL;
 }
 
 static FishSound *
@@ -539,35 +575,61 @@ fs_flac_enc_headers (FishSound * fsound)
   FLAC__stream_encoder_set_channels(fi->fse, fsound->info.channels);
   FLAC__stream_encoder_set_sample_rate(fi->fse, fsound->info.samplerate);
   FLAC__stream_encoder_set_bits_per_sample(fi->fse, BITS_PER_SAMPLE);
+
+#if defined (HAVE_FLAC_1_1_2)
   FLAC__stream_encoder_set_write_callback(fi->fse, fs_flac_enc_write_callback);
   FLAC__stream_encoder_set_metadata_callback(fi->fse, fs_flac_enc_meta_callback);
   FLAC__stream_encoder_set_client_data(fi->fse, fsound);
+#endif
 
   metadata = fs_flac_encode_vorbiscomments (fsound);
   if (metadata != NULL)
     FLAC__stream_encoder_set_metadata (fi->fse, &metadata, 1);
 
   /* FLAC__stream_encoder_set_total_samples_estimate(fi->fse, ...);*/
+
+#if defined (HAVE_FLAC_1_1_2)
   if (FLAC__stream_encoder_init(fi->fse) != FLAC__STREAM_ENCODER_OK)
     return NULL;
+#elif defined (HAVE_FLAC_1_1_3)
+  if (FLAC__stream_encoder_init_stream
+      (fi->fse,
+       fs_flac_enc_write_callback,
+       NULL, /* seek callback */
+       NULL, /* tell callback */
+       fs_flac_enc_meta_callback,
+       fsound
+       ) != FLAC__STREAM_ENCODER_OK)
+     return NULL;
+
+#endif
 
   return fsound;
 }
 
+static long
+fs_flac_encode_fatal (FishSoundFlacInfo *fi, long err)
+{
+  FLAC__stream_encoder_delete (fi->fse);
+  fi->fse = NULL;
+  return err;
+}
+
 static long
 fs_flac_encode_f (FishSound * fsound, float * pcm[], long frames)
 {
   FishSoundFlacInfo *fi = fsound->codec_data;
-  FLAC__int32 *buffer;
+  FLAC__int32 *buffer, *ipcm;
   float * p, norm = (1 << (BITS_PER_SAMPLE - 1));
   long i;
   int j, channels = fsound->info.channels;
 
-#ifdef DEBUG
-  printf("fs_flac_encode_f: IN, frames = %ld\n", frames);
-#endif
+  debug_printf("IN, frames = %ld", frames);
 
-  fi->ipcm = realloc(fi->ipcm, sizeof(FLAC__int32) * channels * frames);
+  if ((ipcm = realloc(fi->ipcm, sizeof(FLAC__int32) * channels * frames)) == NULL)
+    return FISH_SOUND_ERR_OUT_OF_MEMORY;
+
+  fi->ipcm = ipcm;
   buffer = (FLAC__int32*) fi->ipcm;
   for (i = 0; i < frames; i++) {
     for (j = 0; j < channels; j++) {
@@ -581,7 +643,17 @@ fs_flac_encode_f (FishSound * fsound, float * pcm[], long frames)
 
   /* We could have used FLAC__stream_encoder_process() and a more direct
    * conversion loop above, rather than converting and interleaving. */
-  FLAC__stream_encoder_process_interleaved(fi->fse, buffer, frames);
+  if (FLAC__stream_encoder_process_interleaved(fi->fse, buffer, frames) == false) {
+    switch (FLAC__stream_encoder_get_state (fi->fse)) {
+    case FLAC__STREAM_ENCODER_OK:
+    case FLAC__STREAM_ENCODER_UNINITIALIZED:
+      break;
+    case FLAC__STREAM_ENCODER_MEMORY_ALLOCATION_ERROR:
+      return fs_flac_encode_fatal (fi, FISH_SOUND_ERR_OUT_OF_MEMORY);
+    default:
+      return fs_flac_encode_fatal (fi, FISH_SOUND_ERR_GENERIC);
+    }
+  }
 
   fi->packetno++;
 
@@ -592,15 +664,16 @@ static long
 fs_flac_encode_f_ilv (FishSound * fsound, float ** pcm, long frames)
 {
   FishSoundFlacInfo *fi = fsound->codec_data;
-  FLAC__int32 *buffer;
+  FLAC__int32 *buffer, *ipcm;
   float * p = (float*)pcm, norm = (1 << (BITS_PER_SAMPLE - 1));
   long i, length = frames * fsound->info.channels;
 
-#ifdef DEBUG
-  printf("fs_flac_encode_f_ilv: IN, frames = %ld\n", frames);
-#endif
+  debug_printf(1, "IN, frames = %ld", frames);
 
-  fi->ipcm = realloc(fi->ipcm, sizeof(FLAC__int32)*fsound->info.channels*frames);
+  if ((ipcm = realloc(fi->ipcm, sizeof(FLAC__int32)*fsound->info.channels*frames)) == NULL)
+    return FISH_SOUND_ERR_OUT_OF_MEMORY;
+
+  fi->ipcm = ipcm;
   buffer = (FLAC__int32*) fi->ipcm;
   for (i=0; i<length; i++)
     buffer[i] = p[i] * norm;
@@ -608,7 +681,17 @@ fs_flac_encode_f_ilv (FishSound * fsound, float ** pcm, long frames)
   if (fi->packetno == 0)
     fs_flac_enc_headers (fsound);
 
-  FLAC__stream_encoder_process_interleaved(fi->fse, buffer, frames);
+  if (FLAC__stream_encoder_process_interleaved(fi->fse, buffer, frames) == false) {
+    switch (FLAC__stream_encoder_get_state (fi->fse)) {
+    case FLAC__STREAM_ENCODER_OK:
+    case FLAC__STREAM_ENCODER_UNINITIALIZED:
+      break;
+    case FLAC__STREAM_ENCODER_MEMORY_ALLOCATION_ERROR:
+      return fs_flac_encode_fatal (fi, FISH_SOUND_ERR_OUT_OF_MEMORY);
+    default:
+      return fs_flac_encode_fatal (fi, FISH_SOUND_ERR_GENERIC);
+    }
+  }
 
   fi->packetno++;
 
@@ -628,10 +711,6 @@ fs_flac_delete (FishSound * fsound)
   FishSoundFlacInfo * fi = (FishSoundFlacInfo *)fsound->codec_data;
   int i;
 
-#ifdef DEBUG
-  printf("fs_flac_delete: IN\n");
-#endif
-
   if (fsound->mode == FISH_SOUND_DECODE) {
     if (fi->fsd) {
       FLAC__stream_decoder_finish(fi->fsd);
@@ -689,10 +768,7 @@ fs_flac_flush (FishSound * fsound)
 {
   FishSoundFlacInfo * fi = (FishSoundFlacInfo *)fsound->codec_data;
 
-#ifdef DEBUG
-  printf("fs_flac_flush: IN (%s)\n",
-         fsound->mode == FISH_SOUND_DECODE ? "decode" : "encode");
-#endif
+  debug_printf("IN (%s)", fsound->mode == FISH_SOUND_DECODE ? "decode" : "encode");
 
   if (fsound->mode == FISH_SOUND_DECODE) {
     FLAC__stream_decoder_finish(fi->fsd);
@@ -738,6 +814,7 @@ fish_sound_flac_codec (void)
   FishSoundCodec * codec;
 
   codec = (FishSoundCodec *) fs_malloc (sizeof (FishSoundCodec));
+  if (codec == NULL) return NULL;
 
   codec->format.format = FISH_SOUND_FLAC;
   codec->format.name = "Flac (Xiph.Org)";
