@@ -53,6 +53,7 @@ usage (char * progname)
   printf ("  --nasty                   Run with large test parameters\n");
   printf ("  --disable-vorbis          Disable testing of Vorbis codec\n");
   printf ("  --disable-speex           Disable testing of Speex codec\n");
+  printf ("  --disable-flac           Disable testing of Flac codec\n");
   printf ("  --disable-interleave      Disable testing of interleave\n");
   printf ("  --disable-non-interleave  Disable testing of non-interleave\n");
   exit (1);
@@ -61,7 +62,7 @@ usage (char * progname)
 /* For one-time tests, configure these by commandline args */
 static int * test_blocksizes, * test_samplerates, * test_channels;
 static int iter = DEFAULT_ITER;
-static int test_vorbis = HAVE_VORBIS, test_speex = HAVE_SPEEX;
+static int test_vorbis = HAVE_VORBIS, test_speex = HAVE_SPEEX, test_flac = HAVE_FLAC;
 static int test_interleave = 1, test_non_interleave = 1;
 
 static int nasty_blocksizes[] = {128, 256, 512, 1024, 2048, 4096, 0};
@@ -78,8 +79,10 @@ typedef struct {
   int interleave;
   int channels;
   float ** pcm;
-  long frames_in;
-  long frames_out;
+  long actual_frames_in; /* <= actual count of frames encoded */
+  long reported_frames_in; /* <= encoded frameno via fish_sound_frameno() */
+  long actual_frames_out; /* <= actual count of frames decoded */
+  long reported_frames_out; /* <= decoded frameno via fish_sound_frameno() */
 } FS_EncDec;
 
 static int
@@ -87,18 +90,20 @@ decoded_float (FishSound * fsound, float ** pcm, long frames, void * user_data)
 {
   FS_EncDec * ed = (FS_EncDec *) user_data;
 
-  ed->frames_out += frames;
+  ed->actual_frames_out += frames;
+  ed->reported_frames_out = fish_sound_get_frameno (ed->decoder);
 
   return 0;
 }
 
 static int
 decoded_float_ilv (FishSound * fsound, float * pcm[], long frames,
-		   void * user_data)
+                   void * user_data)
 {
   FS_EncDec * ed = (FS_EncDec *) user_data;
 
-  ed->frames_out += frames;
+  ed->actual_frames_out += frames;
+  ed->reported_frames_out = fish_sound_get_frameno (ed->decoder);
 
   return 0;
 }
@@ -128,7 +133,7 @@ fs_fill_square (float * pcm, int length)
 
 static FS_EncDec *
 fs_encdec_new (int samplerate, int channels, int format, int interleave,
-	       int blocksize)
+               int blocksize)
 {
   FS_EncDec * ed;
   FishSoundInfo fsinfo;
@@ -164,8 +169,10 @@ fs_encdec_new (int samplerate, int channels, int format, int interleave,
     }
   }
 
-  ed->frames_in = 0;
-  ed->frames_out = 0;
+  ed->actual_frames_in = 0;
+  ed->reported_frames_in = 0;
+  ed->actual_frames_out = 0;
+  ed->reported_frames_out = 0;
 
   return ed;
 }
@@ -193,42 +200,68 @@ fs_encdec_delete (FS_EncDec * ed)
 
 static int
 fs_encdec_test (int samplerate, int channels, int format, int interleave,
-		int blocksize)
+                int blocksize)
 {
   FS_EncDec * ed;
   char msg[128];
   int i;
+  long expected_frames;
 
   snprintf (msg, 128,
-	    "+ %2d channel %6d Hz %s, %d frame buffer (%s)",
-	    channels, samplerate,
-	    format == FISH_SOUND_VORBIS ? "Vorbis" : "Speex",
-	    blocksize,
-	    interleave ? "interleave" : "non-interleave");
+            "+ %2d channel %6d Hz %s, %d frame buffer (%s)",
+            channels, samplerate,
+            format == FISH_SOUND_VORBIS ? "Vorbis" : (format == FISH_SOUND_FLAC ? "Flac" : "Speex"),
+            blocksize,
+            interleave ? "interleave" : "non-interleave");
   INFO (msg);
   
   ed = fs_encdec_new (samplerate, channels, format, interleave, blocksize);
 
+#if 0
+  fish_sound_comment_add_byname (ed->encoder, "Encoder", "encdec-audio");
+  fish_sound_comment_add_byname (ed->encoder, "Format", msg);
+#endif
+
   for (i = 0; i < iter; i++) {
-    ed->frames_in += blocksize;
-    fish_sound_prepare_truncation (ed->encoder, ed->frames_in,
-				   (i == (iter - 1)));
+    ed->actual_frames_in += blocksize;
+    fish_sound_prepare_truncation (ed->encoder, ed->actual_frames_in,
+                                   (i == (iter - 1)));
     fish_sound_encode (ed->encoder, ed->pcm, blocksize);
+    ed->reported_frames_in = fish_sound_get_frameno (ed->encoder);
   }
 
   fish_sound_flush (ed->encoder);
+  fish_sound_flush (ed->decoder);
+  ed->reported_frames_in = fish_sound_get_frameno (ed->encoder);
 
-  if (ed->frames_in != ed->frames_out) {
+  expected_frames = ed->actual_frames_in;
+  if (format == FISH_SOUND_SPEEX) {
+    expected_frames += (320 - (expected_frames % 320));
+  }
+
+  if (ed->actual_frames_out != expected_frames) {
     snprintf (msg, 128,
-	      "%ld frames encoded, %ld frames decoded",
-	      ed->frames_in, ed->frames_out);
-    if (ed->frames_out < ed->frames_in) {
+              "%ld frames encoded, %ld frames decoded",
+              ed->actual_frames_in, ed->actual_frames_out);
+    if (ed->actual_frames_out < ed->actual_frames_in) {
       FAIL (msg);
     } else {
       WARN (msg);
     }
   }
 
+  expected_frames = ed->reported_frames_in;
+  if (format == FISH_SOUND_SPEEX) {
+    expected_frames += (320 - (expected_frames % 320));
+  }
+
+  if (ed->reported_frames_out != expected_frames) {
+    snprintf (msg, 128,
+              "%ld frames reported in, %ld frames reported out",
+              ed->reported_frames_in, ed->reported_frames_out);
+    WARN (msg);
+  }
+
   fs_encdec_delete (ed);
 
   return 0;
@@ -251,6 +284,8 @@ parse_args (int argc, char * argv[])
       test_vorbis = 0;
     } else if (!strcmp (argv[i], "--disable-speex")) {
       test_speex = 0;
+    } else if (!strcmp (argv[i], "--disable-flac")) {
+      test_flac = 0;
     } else if (!strcmp (argv[i], "--disable-interleave")) {
       test_interleave = 0;
     } else if (!strcmp (argv[i], "--disable-non-interleave")) {
@@ -269,6 +304,7 @@ parse_args (int argc, char * argv[])
 
   if (!test_vorbis) INFO ("* DISABLED testing of Vorbis");
   if (!test_speex) INFO ("* DISABLED testing of Speex");
+  if (!test_flac) INFO ("* DISABLED testing of Flac");
   if (!test_interleave) INFO ("* DISABLED testing of INTERLEAVE");
   if (!test_non_interleave) INFO ("* DISABLED testing of NON-INTERLEAVE");
 }
@@ -288,40 +324,55 @@ main (int argc, char * argv[])
     for (s = 0; test_samplerates[s]; s++) {
       for (c = 0; test_channels[c]; c++) {
 
-	if (test_non_interleave) {
-	  /* Test VORBIS */
-	  if (test_vorbis) {
-	    fs_encdec_test (test_samplerates[s], test_channels[c],
-			    FISH_SOUND_VORBIS, 0, test_blocksizes[b]);
-	  }
-	  
-	  /* Test SPEEX */
-	  if (test_speex) {
-	    if (test_channels[c] <= 2) {
-	      fs_encdec_test (test_samplerates[s], test_channels[c],
-			      FISH_SOUND_SPEEX, 0, test_blocksizes[b]);
-	      
-	    }
-	  }
-	}
+        if (test_non_interleave) {
+          /* Test VORBIS */
+          if (test_vorbis) {
+            fs_encdec_test (test_samplerates[s], test_channels[c],
+                            FISH_SOUND_VORBIS, 0, test_blocksizes[b]);
+          }
+          
+          /* Test SPEEX */
+          if (test_speex) {
+            if (test_channels[c] <= 2) {
+              fs_encdec_test (test_samplerates[s], test_channels[c],
+                              FISH_SOUND_SPEEX, 0, test_blocksizes[b]);
+            }
+         }
 
-	if (test_interleave) {
-	  /* Test VORBIS */
-	  if (test_vorbis) {
-	    fs_encdec_test (test_samplerates[s], test_channels[c],
-			    FISH_SOUND_VORBIS, 1, test_blocksizes[b]);
-	  }
-	  
-	  /* Test SPEEX */
-	  if (test_speex) {
-	    if (test_channels[c] <= 2) {
-	      fs_encdec_test (test_samplerates[s], test_channels[c],
-			      FISH_SOUND_SPEEX, 1, test_blocksizes[b]);
-	      
-	    }
-	  }
-	}
+         /* Test FLAC */
+         if (test_flac) {
+           if (test_channels[c] <= 8) {
+             fs_encdec_test (test_samplerates[s], test_channels[c],
+                             FISH_SOUND_FLAC, 0, test_blocksizes[b]);
+              
+           }
+         }
+        }
 
+        if (test_interleave) {
+          /* Test VORBIS */
+          if (test_vorbis) {
+            fs_encdec_test (test_samplerates[s], test_channels[c],
+                            FISH_SOUND_VORBIS, 1, test_blocksizes[b]);
+          }
+          
+          /* Test SPEEX */
+          if (test_speex) {
+            if (test_channels[c] <= 2) {
+              fs_encdec_test (test_samplerates[s], test_channels[c],
+                              FISH_SOUND_SPEEX, 1, test_blocksizes[b]);
+            }      
+          }
+
+          /* Test FLAC */
+          if (test_flac) {
+            if (test_channels[c] <= 8) {
+              fs_encdec_test (test_samplerates[s], test_channels[c],
+                              FISH_SOUND_FLAC, 1, test_blocksizes[b]);
+              
+            }
+          }
+        }
 
       }
     }
